#!/usr/bin/python
# vim: set fileencoding=utf-8 ts=4 sw=4 tw=79 :

from __future__ import (unicode_literals, absolute_import,
                        division, print_function)

import sys
import urllib2
import xmlrpclib

from urllib import urlencode

NS_BASE = 'http://api.netspeak.org/netspeak3/search'

class Item(dict):
    _defaults = {
        'valid': "yes",
        'autocomplete': "",
        'title': "",
        'subtitle': "",
        'icon': "",
    }
    def __init__(self, **kwargs):
        dict.__init__(self)
        self.update(self._defaults)
        self.update(kwargs)
        self.__dict__ = self

    def __str__(self):
        template = """
        <item uid="{title}" arg="{title}" valid="{valid}"
                autocomplete="{autocomplete}">
            <title>{title}</title>
            <subtitle>{subtitle}</subtitle>
            <icon>{icon}</icon>
        </item>"""
        return template.format(**self)

def generate_xml(l):
    template = """<?xml version="1.0"?>
    <items>
    %s
    </items>"""
    if l:
        return template % "\n".join(str(x) for x in l)
    else:
        return template % str(Item(title='----',
                                   subtitle="The query returns nothing"))

DEFAULT_RESULT = [
    Item(title="how to ? this", subtitle="The ? finds one word."),
    Item(title="see ... works", subtitle="The ... find many words."),
    Item(title="It's [ great well ]", subtitle="The [] compare options"),
    Item(title="and knows #much", subtitle="The # finds similar words."),
    Item(title="{ more show me }", subtitle="The {} check the order."),
]

def http_get(base_url, params):
    handler = urllib2.urlopen(base_url, urlencode(params))
    return handler.read()

def get_results(args):
    params = {
        'topk': '10',
    }
    params['query'] = args
    raw_result = http_get(NS_BASE, params).rstrip()

    results = []
    
    if raw_result:
        total_count = 0
        for line in raw_result.rstrip().split('\n'):
            line = line.strip()
            _, count, string = line.split(None, 2)
            total_count += int(count)
            results.append(Item(title=string, subtitle=int(count)))

        for item in results:
            percent = round(item.subtitle*100/total_count, 2)
            item.subtitle = "%s%%, %s" % (percent, item.subtitle)

    return results

def main(args):
    args = args.strip().lower()
    
    if args == '':
        results = DEFAULT_RESULT
    else:
        results = get_results(args)

    print(generate_xml(results))

if __name__ == '__main__':
    if len(sys.argv) > 1:
        main(sys.argv[1].lower())
